function [results]=recurrenceplot(data,cfg)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Calculates different recurrence based quantities.
%   data: input data, 1xN double or NxN. Input data can either be a time series or
%   an adjacency matrix.
%CONFIGURATION STRUCTURE:
%   cfg.tau: embedding delay, 1x1, int, default: 0 (optimization)
%   cfg.dim: embedding dimension, 1x1, int, default: 2
%   cfg.en: neighbourhood-size in % std of data, 1x1,  double, default: 0
%   cfg.rr: recurrence rate in % of total possible rr, 1x1, double, default: 5
%   cfg.minlengthdet: minimum length of diagonal lines used for determinism
%   quantification, 1x1, int, default: 0 (off)
%   cfg.minlengthlam: minimum length of vertical lines used for laminarity
%   quantification, 1x1, int, default: 0 (off)
%   cfg.minmaxRecPD: minimum and maximum recurrence periods, 1x2, int,
%   default: [0 0]
%   cfg.singlenei: use only nearest neighbour for calculation  of recurrence
%   periods yes/no [1/0], 1x1, int, default: 1
%   cfg.norm: normalize recurrence frequencies by total possible number of
%   recurrences per period bin yes/no [1/0], 1x1, int, default: 0
%   cfg.metric: distance norm 'euclidean' or 'maximum', char, default: 'maximum'
%   cfg.plt: plot results yes/no [1/0], 1x1, int, default: 1
%OUTPUT:
%   results.cfg: configuration structure
%   results.rr: recurrence rate
%   results.det: determinism
%   results.lam: laminarity
%   results.ratio: The ratio between det and rr
%   results.sumdist: thresholded recurrence matrix
%   results.alldist: untresholded recurrence matrix
%   results.ht: recurrence period probabilites as a function of periods
%   results.gaco: generalized autocorrelation as a function of lags
%   results.rpde: recurrence period density entropy
%DEPENDENCIES:
%   amutibin, phasespace, neighsearch
%Author: Immo Weber, 2018
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('       _  __       __    _  ______ _  ___        ');
disp('      / |/ /___   / /   (_)/_  __/(_)/ _ |       ');
disp('     /    // _ \ / /__ / /  / /  / // __ |       ');
disp('    /_/|_/ \___//____//_/  /_/  /_//_/ |_|       ');
disp('                                                 ');
%%%read in parameters%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isfield(cfg,'dim')==1
dim=cfg.dim;
else
    dim=2;
    disp('No embedding dimension specified. Assigning default: 2')
end
if isfield(cfg,'tau')==1
tau=cfg.tau;
else
    tau=0;
    disp('No embedding delay specified. Assigning default: 0')
end
if isfield(cfg,'en')==1
en=cfg.en;
else
    en=0;
    disp('No neighbourhood size specified. Assigning default: 0')
end
if isfield(cfg,'rr')==1
rr=cfg.rr;
else
    rr=5;
    disp('No recurrence rate specified. Assigning default: 5')
end
if isfield(cfg,'minlengthdet')==1
minlengthdet=cfg.minlengthdet;
else
    minlengthdet=0;
    disp('No minimum diagonal line length specified. Assigning default: 0')
end
if isfield(cfg,'minlengthlam')==1
minlengthlam=cfg.minlengthlam;
else
    minlengthlam=0;
    disp('No minimum vertical line length specified. Assigning default: 0')
end
if isfield(cfg,'minmaxRecPD')==1
minRecPD=cfg.minmaxRecPD(1);
maxRecPD=cfg.minmaxRecPD(2);
else
    minRecPD=0;
    maxRecPD=0;
    disp('No range of recurrence periods specified. Assigning defaults: min max')
end
if isfield(cfg,'singlenei')==1
singlenei=cfg.singlenei;
else
    singlenei=1;
    disp('No singlenei parameter specified. Assigning default: 1')
end
if isfield(cfg,'norm')==1
norm=cfg.norm;
else
    norm=0;
    disp('No normalization parameter specified. Assigning default: 0')
end
if isfield(cfg,'metric')==1
if strcmp(cfg.metric,'euclidean')==1    
metric=1;
elseif strcmp(cfg.metric,'maximum')==1    
metric=2;
end
else
    metric=2;
      disp('No distance norm specified! Assigning default: maximum')
end
if isfield(cfg,'plt')==1
plt=cfg.plt;
else
    plt=1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if size(data,1)==1 | size(data,2)==1

%%%if no Tau is provided the first minimum of the auto mutual information%%
%%%is used%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if tau==0
    cfgami.numbin=10;
    cfgami.time=length(data)/2;
    cfgami.plt=0;
    [ resultsami ] = amutibin( data,cfgami);
    tau=resultsami.firstmin;
    disp(['The best lag is probably' ' ' num2str(tau)])
end
%%%Phase-Space reconstruction%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cfgs.dim=dim;
cfgs.tau=tau;
results=phasespace(data,cfgs);                 
space=results.embTS;
%%%calculate distance matrix%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N=length(space);
mat=[];
sumdist=zeros(N,N);
sumdist=neighsearch(space,[1:N]',metric);
distances=sumdist;
%%%calculate neighbourhood-sizes%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if rr~=0
    sortdist=sort(sumdist(:));
    indtemp=(rr/100)*(N^2);
    en=sortdist(floor(indtemp));
else
    groesse=std(data);%max(sumdist(:));
    en=(en*groesse)/100;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sumdist= sumdist>en;

else
    sumdist=data;
    rd=[];
    N=length(sumdist);    
end
n1=sum(sum(sumdist==0));
rd=100*(n1/(N)^2);
alldist=flipud(sumdist);
sumdist=+sumdist;
sumdist=abs(sumdist-1);
%%%%Determinism%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
det=0;
ratio=0;
if minlengthdet>0
    kernel=eye(minlengthdet,minlengthdet);
    det=sum(sum((conv2(double(conv2(sumdist,kernel,'valid')==sum(kernel(:))),kernel,'full'))>0))/N^2;
    ratio=det/rd;
end
lam=0;
if minlengthlam>0
    kernel=ones(minlengthlam,1);
    lam=sum(sum((conv2(double(conv2(sumdist,kernel,'valid')==sum(kernel(:))),kernel,'full'))>0))/N^2;
end


% if minlength>0      
%     tempdiag=0;
%     pattern=['01{1,' num2str(N) '}0'];
% tempsumdist=horzcat(zeros(N,1),sumdist,zeros(N,1));
% tempsumdist=vertcat(zeros(1,N+2),tempsumdist,zeros(1,N+2));
% for diagindex=0:N-1
%     tempdiag=vertcat(tempdiag,diag(tempsumdist,-diagindex));
% end
% tempdiag(1)=[];
%     [s1,e1]=regexp((num2str(tempdiag))',pattern);
%     t1=(e1-s1)-1;
% det=sum(t1(t1>minlength))/sum(t1);
% end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sumdist=tril(sumdist);

%%%generalized autocorrelation%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
indmat=flipud(repmat([1:N]',1,N))-repmat([1:N],N,1);
times=indmat(logical(flipud(sumdist)));
gaco=hist(times,0:N);
gaco=gaco./(N-(0:N));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sumdist=abs(sumdist-1);
sumd=sumdist;
sumdist(end+1,1:length(sumdist))=NaN;
nanpos=find(isnan(sumdist)==1);
temp=find(sumdist==0 | isnan(sumdist)==1);
temp2=diff(temp);
temp2(end+1)=NaN;
[tf,~]=ismember(temp,nanpos);
tf=find(tf==1);
temp2([tf(tf>1)-1])=NaN;
temp2([tf])=NaN;
temp2(temp2==1)=NaN;
if singlenei==1
    temp2([tf])=-1;
    temp2=vertcat(-1,temp2);
    temp2(end)=[];
    temp2(isnan(temp2)==1)=[];
    ind=find(temp2>0);
    ind2=find(temp2(ind) & temp2(ind-1)==-1);
    temp2=temp2(ind(ind2));
end

if  maxRecPD==0
    maxRecPD=length(sumdist);
    minRecPD=1;
end
temp2(isnan(temp2)==1)=[];
ht=hist(temp2,1:length(sumdist));                     
poss=fliplr((1:length(sumdist)));
if norm==1
    ht=ht./poss;
end
htges=ht;
if maxRecPD~=0
    if maxRecPD>length(ht)
        disp('Warning! Maximum recurrence period larger than largest possible period. Assigning maximum...')
        maxRecPD=length(ht);
    end
    ht=ht(minRecPD:maxRecPD);
end

btges=sum(htges);
bt=sum(ht);
htges=htges./btges;
htraw=ht;
ht=ht./bt;
notzeroges=find(htges~=0);
notzero=find(ht~=0);
rpde=-sum(htges(notzeroges).*log(htges(notzeroges)))/(log((notzeroges(end)))); 
alldist=flipud(alldist);
sumdist=sumdist(1:end-1,:);
%%%plot results%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if plt==1
    [r,c] = size(alldist);    
    if exist('distances')
    figure
    ax1=subplot(2,2,1);
    imagesc((1:c),(1:r),distances)
    axis square
    colormap(ax1,jet);
    set(gca,'XTick',0:floor((c+1)/10)+1:(c+1),'YTick',0:floor((c+1)/10)+1:(r+1),...
        'XLim',[0 c+1],'YLim',[0 c+1],'YDir','normal');
    xlabel('state i','fontsize',12)
    ylabel('state j','fontsize',12)
    end
    
    ax2=subplot(2,2,2);
    imagesc((1:c),(1:r),sumdist);
    colormap(ax2,gray);
    axis square
    set(gca,'XTick',0:floor((c+1)/10)+1:(c+1),'YTick',0:floor((c+1)/10)+1:(r+1),...
        'XLim',[0 c+1],'YLim',[0 c+1],'YDir','normal');
    xlabel('state i','fontsize',12)
    ylabel('state j','fontsize',12)
    subplot(2,2,3)
    plot(minRecPD:maxRecPD,ht);
    axis square
    set(gca,'XTick',0:floor(size(ht,2)/10)+1:size(ht,2))
    xlabel('T [samples]')
    ylabel('P(T)')
    subplot(2,2,4)
    plot(gaco)
    axis square
     xlabel('lag [sampels]','fontsize',12)
    ylabel('generalized autocorrelation','fontsize',12)
end
%%%generate output structure%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
results.cfg=cfg;
results.rr=rd;
results.sumdist=sumdist;
if exist('distances')
results.alldist=distances;
end
results.ht=ht;
results.gaco=gaco;
results.rpde=rpde;
results.det=det;
results.lam=lam;
results.ratio=ratio;
results.N=N;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end