function [results]=phasespace(data,varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Embedding of time series in phase-space.
%   data: input data, 1xN, double
%CONFIGURATION STRUCTURE:   
%   cfg.dim: embedding dimension, 1x1, int, default: 2;
%   cfg.tau: embedding delay, 1x1  int, default: 1;
%   cfg.en: neighbourhood size in % of attractor diameter for simple nonlinear noise reduction 
%   (experimental), 1x1, double, default: 0;
%OUTPUT:
%   results.cfg: configuration structure
%   results.embTS: phase-space coordinates, N-(tau*dim*((dim-1)/dim))x dim, double
%   results.residual: extracted noise
%DEPENDENCIES:
%   neighsearch
%Author: Immo Weber, 2018
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% disp('        ______          __             __    ___                    __      __           ');
% disp('       / ____/___ ___  / /_  ___  ____/ /___/ (_)___  ____ _   ____/ /___ _/ /_____ _    ');
% disp('      / __/ / __ `__ \/ __ \/ _ \/ __  / __  / / __ \/ __ `/  / __  / __ `/ __/ __ `/    ');
% disp('     / /___/ / / / / / /_/ /  __/ /_/ / /_/ / / / / / /_/ /  / /_/ / /_/ / /_/ /_/ / _ _ ');
% disp('    /_____/_/ /_/ /_/_.___/\___/\__,_/\__,_/_/_/ /_/\__, /   \__,_/\__,_/\__/\__,_(_|_|_)');
% disp('                                                   /____/                                ');
%%%checking input data%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin==1
    cfg=[];
else
    cfg=varargin{1};
end
% disp('checking input data..')
% if size(data,1)>1 & size(data,2)>1
%     disp('Warning: wrong dimensionality of input data!')
% else 
%     disp('Data OK.')
% end
%%%%%read in parameters%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isfield(cfg,'dim')==1
    m=cfg.dim(1);
else
    m=2;
end
if isfield(cfg,'tau')==1
    tau=cfg.tau(1);
else
    tau=1;
end
if isfield(cfg,'en')==1
    en=cfg.en;
else
    en=0;
end
if isfield(cfg,'backward')==1
    backward=cfg.backward;
else
    backward=1;
end
if size(data,2)>1
    data=data';
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
i=1;
j=1;
space=[];
maxlength=length(data)-(tau*m*((m-1)/m));
if backward==1
data=flipud(data);
end

if strcmp(class(data),'gpuArray')==1
   space=zeros(maxlength,m,'gpuArray');
end

%%%embed  time series%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:maxlength
    space(i,1)=data(i);
    for j=1:m-1
        space(i,j+1)=data(i+tau*j);
    end
end
if backward==1
space=flipud(space);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%simple nonlinear noise reduction%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if en>0    
    N=length(space);
    half=round(m/2);
    mat=[];          
    sumdist=neighsearch(space,[1:N]');          
    maxsize=max(sumdist(:));
    en=en*maxsize/100;
    adj=cell(1,length(data));
    for i=1:length(space)
        adj{i}=find(sumdist(i,:)<=en);
        spacecleaned(i,half)=mean(space(adj{i}(:),half));
    end 
    results.residual=space(:,half)-spacecleaned(:,half);
    space=spacecleaned;
end

%%%output structure%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
results.cfg=cfg;
results.embTS=space;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end